use std::{
    io::{self, Read},
    thread,
    time::{Duration, Instant},
};

use arg::Args;
use bitvec::{order::Msb0, view::AsBits};
use clap::Parser;
use colored::Colorize;
use console::Term;
use rapidriter_cat::next_frame;

const WIDTH: usize = 96;
const HEIGHT: usize = 38;

mod arg;

fn main() {
    let args = Args::parse();

    // clear screen
    print!("\x1B[2J");

    if args.interactive {
        interactive_mode();
    } else {
        non_interactive_mode();
    }
}

fn interactive_mode() {
    let mut frame = [0; 456];

    let mut i = 0;

    let term = Term::stdout();

    loop {
        let has_next_frame = i < 99 && unsafe { next_frame(i, frame.as_mut_ptr()) } == 0;

        draw_frame(&frame);
        print!("Frame: {i}                  ");
        if !has_next_frame {
            print!(" (Last frame)");
        }
        println!("\np for previous frame, n for next frame, q to quit");

        loop {
            match term.read_char().expect("stdin error") {
                'n' if has_next_frame => {
                    i += 1;
                    break;
                }
                'p' if i > 0 => {
                    i -= 1;
                    break;
                }
                'q' => return,
                _ => {}
            }
        }
    }
}

fn non_interactive_mode() {
    let mut frame = [0; 456];

    let start = Instant::now();
    for i in 0..100u16 {
        let target = start + Duration::from_millis(u64::from(i) * 100);
        let now = Instant::now();
        if now < target {
            thread::sleep(target - now);
        }

        if unsafe { next_frame(i, frame.as_mut_ptr()) } == 1 {
            break;
        }

        draw_frame(&frame);
        println!("Frame: {}", i);
    }
}

fn draw_frame(frame: &[u8; 456]) {
    // reset cursor position to (1,1)
    print!("\x1B[1;1H");

    let bv = frame.as_bits::<Msb0>();

    // top part of border
    for _ in 0..(WIDTH + 2) {
        print!("{}", " ".on_white());
    }
    println!();

    for y in 0..HEIGHT {
        print!("{}", " ".on_white());

        for x in 0..WIDTH {
            if bv[x + y * WIDTH] {
                print!("{}", " ".on_red());
            } else {
                print!(" ");
            }
        }

        print!("{}", " ".on_white());

        println!();
    }

    // bottom part of border
    for _ in 0..(WIDTH + 2) {
        print!("{}", " ".on_white());
    }

    println!();
}
